/* $Workfile: OtxT1E1D.h $   $Revision: 46 $    $Modtime: 3/02/01 6:53p $  */
#ifndef OTX_T1E1_DEF_H
#define OTX_T1E1_DEF_H
/*******************************************************************************

  @doc PublicHwApi OtxT1E1Def

  Copyright (C) 1998-2001 Odin TeleSystems Inc., All Rights Reserved

  OTX Generic Hardware Driver API

  @module
  OtxT1E1D | Constants, Macros, and Types for the T1/E1 Line Interface Device Functions

  @contents2
  OtxT1E1D.h |

  @index enum struct type | OtxT1E1Def

  @contents2
  OTX T1/E1 Line Interface Constants, Macros, and Types |

  @head3 OtxT1E1XXX, OtxT1XXX, and OtxE1XXX Macro, Constant, and Type Reference |

  @index enum struct type | OtxT1E1D

*******************************************************************************/

#include "OtxType.h"

#ifdef __cplusplus
  extern "C" {
#endif

#ifdef __linux__
#pragma pack(4)
#else
#pragma pack(push,4)
#endif


/*******************************************************************************
  @enum 
    T1E1 Attributes
*******************************************************************************/
enum OtxT1E1AttributesE {
  OTX_ATTR_T1E1_TX_PULSE_MASK_SUB_PULSE_0    = OTX_ATTR_END + 0,  // @emem (OTX_ATTRIB_UINT32) Read/Write. First (of four) sub pulse shape value of the transmit pulse mask. This 5-bit value defines the level of the analog voltage in steps of about 80mV (for this sub pulse) on transmit pins of the E1T1 span.
  OTX_ATTR_T1E1_TX_PULSE_MASK_SUB_PULSE_1    = OTX_ATTR_END + 1,  // @emem (OTX_ATTRIB_UINT32) Read/Write. Second (of four) sub pulse shape value of the transmit pulse mask. This 5-bit value defines the level of the analog voltage in steps of about 80mV (for this sub pulse) on transmit pins of the E1T1 span.
  OTX_ATTR_T1E1_TX_PULSE_MASK_SUB_PULSE_2    = OTX_ATTR_END + 2,  // @emem (OTX_ATTRIB_UINT32) Read/Write. Third (of four) sub pulse shape value of the transmit pulse mask. This 5-bit value defines the level of the analog voltage in steps of about 80mV (for this sub pulse) on transmit pins of the E1T1 span.
  OTX_ATTR_T1E1_TX_PULSE_MASK_SUB_PULSE_3    = OTX_ATTR_END + 3,  // @emem (OTX_ATTRIB_UINT32) Read/Write. Forth (of four) sub pulse shape value of the transmit pulse mask. This 5-bit value defines the level of the analog voltage in steps of about 80mV (for this sub pulse) on transmit pins of the E1T1 span.

  OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS  = OTX_ATTR_END + 4,  // @emem (OTX_ATTRIB_BOOL) Read/Write. If set to OTX_TRUE, the ABCD bit devices 
                                                                  // (OTX_LDEVICE_T1_ROB_BIT_ACCESS and OTX_LDEVICE_E1_ABCD_BIT_ACCESS) will report changes 
                                                                  // in the ABCD bit code on a channelized basis (using the OTX_T1E1_EC_BITROB_SIGNALING event). 
                                                                  // The <e OtxEventDataS.m_nParam> field in the OTX_T1E1_EC_BITROB_SIGNALING event will contain
                                                                  // channel number in bit12-bit8 and the ABCD bit code in bit3-bit0 (A=bit3,B=bit2,C=bit1,D=bit0)
                                                                  // <nl> The default is OTX_FALSE and OTX_LDEVICE_E1_ABCD_BIT_ACCESS event then contains ABCD bit
                                                                  // information as described by <t OtxE1ABCDBitDataS> and <t OtxT1RobBitDataS>.

  OTX_ATTR_T1E1_S_BIT_CHANGE_EVENTS          = OTX_ATTR_END + 5,  // @emem (OTX_ATTRIB_BOOL) Read/Write. If set to OTX_TRUE, OTX_LDEVICE_E1_S_BIT_ACCESS will generate
                                                                  // an OTX_T1E1_EC_SA_BITS_CHANGED event for each change in the received Sa bits.

  OTX_ATTR_T1E1_SECONDS_INTERRUPT            = OTX_ATTR_END + 6,  // @emem (OTX_ATTRIB_BOOL) Read/Write. Default is OTX_FALSE. If set to OTX_TRUE, the device will generate 
                                                                  // <e OtxT1E1EventCodesE.OTX_T1E1_EC_SEC> event once every second. This event can be utilized
                                                                  // as a "heartbeat" watch dog from the device to verify that interrupts from this device are
                                                                  // operational.

  OTX_ATTR_T1E1_AUTOMATIC_RESET              = OTX_ATTR_END + 7,  // @emem (OTX_ATTRIB_BOOL) Read/Write. Default is OTX_TRUE. If set to OTX_TRUE, the device will generate 
                                                                  // monitor the contents of its register and if an unexpected discrepancy is encountered, the device will reset itself
                                                                  // and reconfigure itself to the last known good state. If the automatic reset is triggered, the device 
                                                                  // generated an <e OtxT1E1EventCodesE.OTX_T1E1_EC_AUTO_RESET>
                                                                  // In case a scenario of cyclic resets occurs, this attribute will be set to OTX_FALSE after 10 reset attempts, to avoid
                                                                  // a system lockup.

  OTX_ATTR_T1E1_END                          = OTX_ATTR_END + 8
};

typedef enum OtxT1E1AttributesE OtxT1E1AttributesE;




/*******************************************************************************
  @enum
    OTX_DEVICE_LI_T1E1,
    OTX_LDEVICE_T1_ROB_BIT_ACCESS,
    OTX_LDEVICE_T1_DL_BIT_ACCESS,
    OTX_LDEVICE_E1_S_BIT_ACCESS and
    OTX_LDEVICE_E1_ABCD_BIT_ACCESS  
    Result Codes
*******************************************************************************/
enum OtxT1E1ResultCodesE {
  OTX_T1E1_RES_BAD_AIS_DETECTION_OPT    = OTX_RES_END +  0 ,  // @emem Invalid parameter value passed for AIS Detection.        
  OTX_T1E1_RES_BAD_ALARM_TYPE           = OTX_RES_END +  1 ,  // @emem Invalid parameter value passed for Alarm Type.           
  OTX_T1E1_RES_BAD_CLOCK_MODE           = OTX_RES_END +  2 ,  // @emem Invalid parameter value passed for Clock Mode.           
  OTX_T1E1_RES_BAD_FRAME_FORMAT         = OTX_RES_END +  3 ,  // @emem Invalid parameter value passed for Frame Format.         
  OTX_T1E1_RES_BAD_HDB3_ERROR_OPT       = OTX_RES_END +  4 ,  // @emem Invalid parameter value passed for HDB3 Error Type.      
  OTX_T1E1_RES_BAD_LOOP_TYPE            = OTX_RES_END +  5 ,  // @emem Invalid parameter value passed for Loop Mode.            
  OTX_T1E1_RES_BAD_REGAIN_M_FRAME_OPT   = OTX_RES_END +  6 ,  // @emem Invalid parameter value passed for Regain Multiframe.    
  OTX_T1E1_RES_BAD_REMOTE_ALARM_OPT     = OTX_RES_END +  7 ,  // @emem Invalid parameter value passed for Remote Alarm.         
  OTX_T1E1_RES_BAD_RESYNC_OPT           = OTX_RES_END +  8 ,  // @emem Invalid parameter value passed for Resynchronization.    
  OTX_T1E1_RES_BAD_RX_EQUALIZER_OPT     = OTX_RES_END +  9 ,  // @emem Invalid parameter value passed for Receiver Equalizer.   
  OTX_T1E1_RES_BAD_RX_FRAME_FORMAT      = OTX_RES_END + 10 ,  // @emem Invalid parameter value passed for Receiver Frame Format.
  OTX_T1E1_RES_BAD_RX_LINE_CODE         = OTX_RES_END + 11 ,  // @emem Invalid parameter value passed for Reciever Line Code.   
  OTX_T1E1_RES_BAD_RX_REM_ALRM_FORMAT   = OTX_RES_END + 12 ,  // @emem Invalid parameter value passed for Receiver Remote Alarm.
  OTX_T1E1_RES_BAD_SIGNALING_MODE       = OTX_RES_END + 13 ,  // @emem Invalid parameter value passed for Signalling Mode.      
  OTX_T1E1_RES_BAD_TX_FRAME_FORMAT      = OTX_RES_END + 14 ,  // @emem Invalid parameter value passed for Transmit Frame Format.
  OTX_T1E1_RES_BAD_TX_LINE_CODE         = OTX_RES_END + 15  , // @emem Invalid parameter value passed for Transmit Line Code.   
  OTX_T1E1_RES_BAD_TX_POWER_OPT         = OTX_RES_END + 16  , // @emem Invalid parameter value passed for Transmit Power.       
  OTX_T1E1_RES_BAD_TX_REM_ALRM_FORMAT   = OTX_RES_END + 17  , // @emem Invalid parameter value passed for Transmit Remote Alarm.
  OTX_T1E1_RES_L1_DOWN                  = OTX_RES_END + 18  , // @emem Loss Of Signal. Layer 1 is down.                         
  OTX_T1E1_RES_L1_OK                    = OTX_RES_END + 19  , // @emem E1/T1 receiver is receiving valid signals. Layer 1 is OK.
  OTX_T1E1_RES_NOT_SETUP                = OTX_RES_END + 20  , // @emem The device is not setup to support this action.          
  OTX_T1E1_RES_TX_BUSY                  = OTX_RES_END + 21  , // @emem Transmitter is busy sending a frame.                     
  OTX_T1E1_RES_WRONG_CONTEXT            = OTX_RES_END + 22  , // @emem Wrong context for this action.                           
  OTX_T1E1_RES_UNKNOWN_STATUS_CODE      = OTX_RES_END + 23  , // @emem Invalid event code passed to this function.              
  OTX_T1E1_RES_UNKNOWN_MODE             = OTX_RES_END + 24  , // @emem The device is not setup in a valid mode.                 
  OTX_T1E1_RES_INVALID_CHANNEL          = OTX_RES_END + 25  , // @emem Invalid channel.                                         
  OTX_T1E1_RES_UNSUPPORTED_VER          = OTX_RES_END + 26  , // @emem Unsupported Device version.                              
  OTX_T1E1_RES_END                      = OTX_RES_END + 27
};

typedef enum OtxT1E1ResultCodesE OtxT1E1ResultCodesE;


/*******************************************************************************
  @enum
    OTX_DEVICE_LI_T1E1,
    OTX_LDEVICE_T1_ROB_BIT_ACCESS,
    OTX_LDEVICE_E1_S_BIT_ACCESS AND
    OTX_LDEVICE_E1_ABCD_BIT_ACCESS  
    Event Codes

    Note that for line interfaces these messages can have a different
    meaning depending on whether the Line Interfaces (LIs) are configured to
    E1 or T1 mode.

*******************************************************************************/
enum OtxT1E1EventCodesE {
  OTX_T1E1_EC_T8MS_ISF             = 0x00, // @emem  E1: Receive Timeout 8 MSec
                                           // <nl>   T1: Incorrect Sync Format

  OTX_T1E1_EC_CASC_RSC             = 0x01, // @emem  E1: Received CAS information Changed
                                           // <nl>   T1: Received Signalling information Changed

  OTX_T1E1_EC_CRC4_CRC6            = 0x02, // @emem  E1: Receive CRC4 error
                                           // <nl>   T1: Receive CRC6 error

  OTX_T1E1_EC__CASE                = 0x03, // @emem  E1: -
                                           // <nl>   T1: Transmit CAS register Empty

  OTX_T1E1_EC_RDO                  = 0x04, // @emem  E1: Receive Data Overflow
                                           // <nl>   T1: Receive Data Overflow

  OTX_T1E1_EC_XDU                  = 0x05, // @emem  E1: Transmit Data Underrun
                                           // <nl>   T1: Transmit Data Underrun

  OTX_T1E1_EC_XLSC                 = 0x06, // @emem  E1: Transmit Line Status Change
                                           // <nl>   T1: Transmit Line Status Change

  OTX_T1E1_EC_FAR                  = 0x07, // @emem  E1: Frame Alignment Recovery
                                           // <nl>   T1: Frame Alignment Recovery

  OTX_T1E1_EC_LFA                  = 0x08, // @emem  E1: Loss of Frame Alignment
                                           // <nl>   T1: Loss of Frame Alignment

  OTX_T1E1_EC_MFAR                 = 0x09, // @emem  E1: MultiFrame Alignment Recovery
                                           // <nl>   T1: MultiFrame Alignment Recovery

  OTX_T1E1_EC_T400MS_LMFA          = 0x0A, // @emem  E1: Receive Timeout 400 MSec
                                           // <nl>   T1: Loss of MultiFrame Alignment

  OTX_T1E1_EC_AIS                  = 0x0B, // @emem  E1: Alarm Indication Signal (Blue Alarm)
                                           // <nl>   T1: Alarm Indication Signal (Blue Alarm)

  OTX_T1E1_EC_LOS                  = 0x0C, // @emem  E1: Loss Of Signal (Red Alarm)
                                           // <nl>   T1: Loss Of Signal (Red Alarm)

  OTX_T1E1_EC_RAR                  = 0x0D, // @emem  E1: Remote Alarm Recovery
                                           // <nl>   T1: Remote Alarm Recovery

  OTX_T1E1_EC_RA                   = 0x0E, // @emem  E1: Remote Alarm (Yellow Alarm)
                                           // <nl>   T1: Remote Alarm (Yellow Alarm)

  OTX_T1E1_EC_LMFA16_XSLP          = 0x0F, // @emem  E1: Loss of MultiFrame Alignment ts16
                                           // <nl>   T1: Transmit SLiP indication

  OTX_T1E1_EC_AIS16_               = 0x10, // @emem  E1: Alarm Indication Signal ts16 status change
                                           // <nl>   T1: -

  OTX_T1E1_EC_RA16_LLBSC           = 0x11, // @emem  E1: Remote Alarm ts16 status change
                                           // <nl>   T1: Line Loop Back Status Change

  OTX_T1E1_EC_API_                 = 0x12, // @emem  E1: Auxuillary Pattern Indication
                                           // <nl>   T1: -

  OTX_T1E1_EC_SLN                  = 0x13, // @emem  E1: Receive Slip Negative
                                           // <nl>   T1: Receive Slip Negative

  OTX_T1E1_EC_SLP                  = 0x14, // @emem  E1: Receive Slip Positive
                                           // <nl>   T1: Receive Slip Positive

  OTX_T1E1_EC_ACTIVE               = 0x15, // @emem  E1: Physical link ACTIVE indication
                                           // <nl>   T1: Physical link ACTIVE indication

  OTX_T1E1_EC_DEACTIVE             = 0x16, // @emem  E1: Physical link DEACTIVE indication
                                           // <nl>   T1: Physical link DEACTIVE indication

  OTX_T1E1_EC_XLS                  = 0x17, // @emem  E1: Transmit Line Short
                                           // <nl>   T1: Transmit Line Short

  OTX_T1E1_EC_XLO                  = 0x18, // @emem  E1: Transmit Line Open
                                           // <nl>   T1: Transmit Line Open

  OTX_T1E1_EC_XPR                  = 0x19, // @emem  Transmit Pool Ready

  OTX_T1E1_EC_SEC                  = 0x1a, // @emem  Second

  OTX_T1E1_EC_ES                   = 0x1b, // @emem  Errored second

  OTX_T1E1_EC_RME                  = 0x1c, // @emem  Receive Message End

  OTX_T1E1_EC_RPF                  = 0x1d, // @emem  Receive Pool Full

  OTX_T1E1_EC_XSP                  = 0x1e, // @emem  E1: Transmit Slip Positive
                                           // <nl>   T1: Transmit Slip Positive

  OTX_T1E1_EC_XSN                  = 0x1f, // @emem  E1: Transmit Slip Negative
                                           // <nl>   T1: Transmit Slip Negative

  OTX_T1E1_EC_BITROB_SIGNALING     = 0x20, // @emem  E1: -
                                           // <nl>   T1: BitRob Signaling Data

  OTX_T1E1_EC_FEC                  = 0x21, // @emem  E1: Frame Error Count
                                           // <nl>   T1: Frame Error Count

  OTX_T1E1_EC_CVC                  = 0x22, // @emem  E1: Code Violation Count
                                           // <nl>   T1: Code Violation Count

  OTX_T1E1_EC_EBC                  = 0x23, // @emem  E1: E-bit Error Count
                                           // <nl>   T1: Errored Block Count

  OTX_T1E1_EC_CEC1                 = 0x24, // @emem  E1: CRC Error Count
                                           // <nl>   T1: CRC Error Count

  OTX_T1E1_EC_CEC2                 = 0x25, // @emem  E1: CRC Error Count (reported from TE via Sa6-bit)
                                           // <nl>   T1: -

  OTX_T1E1_EC_CEC3                 = 0x26, // @emem  E1: CRC Error Count (detected at T reference point via Sa6-bit)
                                           // <nl>   T1: -

  OTX_T1E1_EC_APIR                 = 0x27, // @emem  E1: Auxiliary Pattern Indication (Blue Alarm) Recovery
                                           // <nl>   T1: -

  OTX_T1E1_EC_LOSR                 = 0x28, // @emem  E1: Loss Of Signal (Red Alarm) Recovery
                                           // <nl>   T1: Loss Of Signal (Red Alarm) Recovery

  OTX_T1E1_EC_LMFAR16              = 0x29, // @emem  E1: Loss of MultiFrame Alignment ts16 Recovery
                                           // <nl>   T1: -

  OTX_T1E1_EC_AISR                 = 0x2a, // @emem  E1: Alarm Indication Signal (Blue Alarm) Recovery
                                           // <nl>   T1: Alarm Indication Signal (Blue Alarm) Recovery

  OTX_T1E1_EC_SA_BITS_CHANGED      = 0x2b, // @emem  E1: SA-Bits (4-8) changed. Bit0 is Sa bit from Frame1, Bit0 is Sa bit from Frame3, ... , Bit7 is Sa bit from Frame15. Bit11-Bit8 contains the Sa number (4-8).
                                           // <nl>   T1: -

  OTX_T1E1_EC_DL_BITS_RECEIVED     = 0x2c, // @emem  E1: -
                                           // <nl>   T1: DL-Bits Received:
                                           // <nl> Mode <e OtxT1FrameFormatE.OTX_T1_FF_F12>:
                                           // <nl> Bit0 is received in frame 2 and Bit7 in frame 12.
                                           // <nl> Remaining bits are zero.
                                           // <nl> 
                                           // <nl> Mode <e OtxT1FrameFormatE.OTX_T1_FF_ESF>:
                                           // <nl> Bit0 is received in frame  1 and Bit7  in frame 15.
                                           // <nl> Bit8 is received in frame 17 and Bit15 in frame 23.
                                           // <nl> Remaining bits are zero.
                                           // <nl> 
                                           // <nl> Mode <e OtxT1FrameFormatE.OTX_T1_FF_F72>:
                                           // <nl> Bit0  is received in frame 26 and Bit7  in frame 40
                                           // <nl> Bit8  is received in frame 42 and Bit15 in frame 56.
                                           // <nl> Bit16 is received in frame 58 and Bit23 in frame 72.
                                           // <nl> Remaining bits are zero.

  OTX_T1E1_EC_DL_BITS_SENT         = 0x2d, // @emem  E1: -
                                           // <nl>   T1: DL-Bits Sent. The function <f OtxT1SetDlBitData> needs to
                                           // be called to provide more DL-bits to be transmitted by the T1E1 device.
                                           // If <f OtxT1SetDlBitData> is not called before the start of the next
                                           // multiframe then the DL-Bit device will continue to transmit the same DL-bits 
                                           // (as were provided by the most recent call to <f OtxT1SetDlBitData>.
                                           // The bits should be formatted in the same way as 
                                           // in <e OtxT1E1EventCodesE.OTX_T1E1_EC_DL_BITS_RECEIVED>.

  OTX_T1E1_EC_AUTO_DISABLED        = 0x2e, // @emem  E1: Fatal error has occurred. The device is automatically disabled.
                                           // <nl>       Try reviving the device with <f OtxDrvReset> and <f OtxDrvEnable>.
                                           // <nl>   T1: Fatal error has occurred. The device is automatically disabled.
                                           // <nl>       Try reviving the device with <f OtxDrvReset> and <f OtxDrvEnable>.

  OTX_T1E1_EC_AUTO_RESET           = 0x2f, // @emem  E1: Serious error has occurred. The device has performed an automatic self-reset.
                                           // @emem  T1: Serious error has occurred. The device has performed an automatic self-reset.

  OTX_T1E1_EC_END_OF_EVENT_CODES

};

typedef enum OtxT1E1EventCodesE OtxT1E1EventCodesE;


/*******************************************************************************
  OTX_DEVICE_LI_T1E1 Common Types
*******************************************************************************/

/*******************************************************************************
  @enum
    Operating modes for the T1/E1 Line Interface (Li) Devices
  @xref
    <f OtxT1E1SetLiMode>
*******************************************************************************/
enum OtxT1E1LiModeE {
  OTX_T1E1_MODE_NO = 0,    // @emem The line interface has not been configured
  OTX_T1E1_MODE_T1 = 1,    // @emem The line interface is configured to operate in T1 mode.
  OTX_T1E1_MODE_E1 = 2     // @emem The line interface is configured to operate in E1 mode.
};

typedef enum OtxT1E1LiModeE OtxT1E1LiModeE;



/*******************************************************************************
  @enum
    Layer 1 Status
  @xref
    <f OtxT1E1GetLineStatus>
*******************************************************************************/
enum OtxT1E1Layer1StatusE {
  OTX_T1E1_LAYER1_DOWN = 0,  // @emem Loss of Signal (LOS) or Loss Of Frame Alignment (LFA) is present
  OTX_T1E1_LAYER1_OK   = 1   // @emem E1/T1 signal present (No Loss of Signal and no Loss Of Frame Alignment) 
};

typedef enum OtxT1E1Layer1StatusE OtxT1E1Layer1StatusE;



enum OtxT1E1ClkModeE {
  OTX_T1E1_CLOCK_NO = 0,       // No Clock mode has been set
  OTX_T1E1_CLOCK_MASTER = 1,   // Clock Master
  OTX_T1E1_CLOCK_SLAVE = 2     // Clock Slave
};

typedef enum OtxT1E1ClkModeE OtxT1E1ClkModeE;



/*******************************************************************************
  @enum
    Alarms types that can be send towards the Remote End
  @xref
    <f OtxT1E1AlarmOn>
    <f OtxT1E1AlarmOff>
*******************************************************************************/
enum OtxT1E1AlarmTypeE {
  OTX_T1E1_ALARM_AUXP       = 0,  // @parm Alarm Type Auxilliary Pattern (AUXP)
  OTX_T1E1_ALARM_AIS        = 1,  // @parm Alarm Indication Signal (AIS, Blue Alarm)
  OTX_T1E1_ALARM_RAI        = 2,  // @parm Remote Alarm Indication (RAI, Yellow Alarm) 
  OTX_T1E1_ALARM_SIMULATION = 3,  // @parm Initiates internal error simulation of AIS,
                                  // loss of signal, loss of synchronization,
                                  // auxilliary pattern indication, slip, framing
                                  // errors, CRC errors, and code violations.
};

typedef enum OtxT1E1AlarmTypeE OtxT1E1AlarmTypeE;


/*******************************************************************************
  @enum
    Line Interface loop type
  @xref
    <f OtxT1E1SetLineLoop>
*******************************************************************************/
enum OtxT1E1LoopE {
  OTX_T1E1_LOOPS_OFF                    = 0,   // @emem All Line Loops off
  OTX_T1E1_LINE_LOOP                    = 1,   // @emem Line Loop (Payload Loopback of the external highway). Looped data will pass through the entire receiver including the wander and jitter compensation.
  OTX_T1E1_REMOTE_ON_LOOP_JITTER_ATTEN  = 2,   // @emem Remote Loop with Jitter Attenuation On (Loopback of the external highway bypassing the framer)
  OTX_T1E1_REMOTE_OFF_LOOP_JITTER_ATTEN = 3,   // @emem Remote Loop with Jitter Attenuation Off (Loopback of the external highway bypassing the framer)
  OTX_T1E1_LOCAL_LOOP                   = 4,   // @emem Local Loop. The receiver on the external highway is disconnected. The internal highway is looped back to the time-space switch. 
};

typedef enum OtxT1E1LoopE OtxT1E1LoopE;



/*******************************************************************************
  T1 Specfic Configuration Types
*******************************************************************************/


/*******************************************************************************
   @enum
     T1 Signalling Modes
   @xref
     <t OtxT1ConfigOptionsS>
     <f OtxT1SetConfiguration>
*******************************************************************************/
enum OtxT1SignallingModeE {
  OTX_T1_SM_CCS      = 0,    // @emem Common Channel Signalling
  OTX_T1_SM_CAS_CC   = 1,    // @emem Channel Associated Signalling (Common channel)
  OTX_T1_SM_CAS_BR   = 2     // @emem Channel Associated Signalling (Bit Robbing)
};

typedef enum OtxT1SignallingModeE OtxT1SignallingModeE;


/*******************************************************************************
   @enum
     T1 Line Codes
   @xref
     <t OtxT1ConfigOptionsS>
     <f OtxT1SetConfiguration>
*******************************************************************************/
enum OtxT1LineCodeE {
  OTX_T1_LC_AMI   = 0,   // @emem Alternative Mark Inversion (AMI) Line Coding
  OTX_T1_LC_B8ZS  = 1    // @emem Bipolar 8 Zero Subsititution (B8ZS) Line Coding
};

typedef enum OtxT1LineCodeE OtxT1LineCodeE;



/*******************************************************************************
   @enum
     T1 Frame Formats
   @xref
     <t OtxT1ConfigOptionsS>
     <f OtxT1SetConfiguration>
*******************************************************************************/
enum OtxT1FrameFormatE {
  OTX_T1_FF_F12 = 0,  // @parm 12-frame multiframe format (F12, D3/4, SF)
  OTX_T1_FF_F4  = 1,  // @parm 4-frame multiframe format (F4)
  OTX_T1_FF_ESF = 2,  // @parm 24-frame multiframe format (F24, ESF)
  OTX_T1_FF_F72 = 3,  // @parm 72-frame multiframe format (F72, SLC96, remote switch mode)
};

typedef enum OtxT1FrameFormatE OtxT1FrameFormatE;


/*******************************************************************************
   @enum
     Set the condition when the Loss of Frame Alignment (LFA) will be reported.
   @comm
     LFA can be reported if 2 out of 4, 5, or 6 framing bits have been detected to be
     incorrect.
   @xref
     <t OtxT1ConfigOptionsS>
     <f OtxT1SetConfiguration>
*******************************************************************************/
enum OtxT1LfaSensitivityE {
  OTX_T1_LFA_2_OUT_OF_4 = 0x00,    // @parm 2 out of 4 framing bits incorrect
  OTX_T1_LFA_2_OUT_OF_5 = 0x08,    // @parm 3 out of 4 framing bits incorrect
  OTX_T1_LFA_2_OUT_OF_6 = 0x10,    // @parm 4 out of 4 framing bits incorrect
};

typedef enum OtxT1LfaSensitivityE OtxT1LfaSensitivityE;


/*******************************************************************************
   @enum
     Select the Remote Alarm Indication (Yellow Alarm) format to be expected
     in Receive direction with the F12 and ESF Frames.

   @comm
     If Format A is selected, with F12 Frame Format the bit #2 in every channel is
     set to `0'. With ESF Frame Format a pattern on `1111 1111 0000 0000 ...' is sent in
     the data link channel. If Format B is selected, with F12 Frame Format the Frame
     Synchronization (FS) bit of frame 12 is set to `0'. With ESF Frame Format the bit
     position 2 in every time-slot is set to `0'. With F4 and F72 frames formats the
     Yellow alarm is indicated in both formats by forcing the bit
     position 2 to `1' in every speech channel.

   @xref
     <t OtxT1ConfigOptionsS>
     <f OtxT1SetConfiguration>

*******************************************************************************/
enum OtxT1RemoteAlarmFormatE {
  OTX_T1_RA_YELLOW_A = 0,  // @emem F12: bit2 = 0  in every channel
                           // <nl> ESF: pattern '1111 1111 0000 0000' in data link channel
  OTX_T1_RA_YELLOW_B = 1,  // @emem F12: FS bit of frame 12
                           // <nl> ESF: bit2 = 0  in every channel
};

typedef enum OtxT1RemoteAlarmFormatE OtxT1RemoteAlarmFormatE;


/*******************************************************************************
  @struct
    T1 Line Interface Configuration Options
  @xref
   <f OtxT1SetConfiguration>
   <f OtxT1GetConfiguration>
*******************************************************************************/
struct OtxT1ConfigOptionsS {
  OtxT1SignallingModeE    eSignalingMode;  // @field Signalling Mode. See type <t OtxT1SignallingModeE>
  OtxT1LineCodeE          eTxLineCode;     // @field Line Coding in Transmit Direction. See type <t OtxT1LineCodeE>
  OtxT1LineCodeE          eRxLineCode;     // @field Line Coding in Receive Direction
  OtxT1FrameFormatE       eFrameFormat;    // @field Frame Format. See type <t OtxT1FrameFormatE>
  OTX_BOOL                bCRC6Enable;     // @field Set whether CRC6 checking and generation will be enabled when using
                                           // the ESF frame format. When disabled, the receive will not perform the
                                           // CRC6 check and the CRC6 bits are set to `1' to the transmit direction.
  OtxT1RemoteAlarmFormatE eTxRemoteAlarmFormat; // @field Transmit Remote Alarm Format. See type <t OtxT1RemoteAlarmFormatE>
  OtxT1RemoteAlarmFormatE eRxRemoteAlarmFormat; // @field Receive Remote Alarm Format. See type <t OtxT1RemoteAlarmFormatE>
  OTX_BOOL                bAutoResync;     // @field If OTX_TRUE, the Framer will automatically re-synchronize after
                                           // loss of synchronization. If OTX_FALSE the Framer will not synchronize
                                           // automatically but a re-synchronization procedure must be specifically
                                           // initiated by the software.
  OTX_BOOL                bAutoCrcBitInversion; // @field Valid only for ESF frame format. For other frame format this value
                                           // is ignored.
                                           // <nl>If OTX_TRUE, all CRC6 bits of one outgoing sub-multiframe are
                                           // inverted in case a CRC6 error is flagged for the previous received 
                                           // sub-multiframe.
  OtxT1LfaSensitivityE    eLfaSensitivity; // @field Loss of Frame Alignment Sensitivity. See type <t OtxT1LfaSensitivityE>
  OTX_BOOL                bAutoRemoteAlarm;// @field If OTX_TRUE, a Remote Alarm (Yellow Alarm) will be sent automatically
                                           // in the outgoing data stream when the receiver is out of synchronization.
                                           // The alarm will also be automatically removed when the receiver returns to
                                           // the synchronized state.
  OTX_UINT8               nLosSensitivity; // @field Sets the sensitivity level for the Loss Of Signal (LOS) detection.
                                           // A LOS alarm will be generated if the incoming data stream has no
                                           // transitions during programmable number (N) of consecutive pulse periods.
                                           // The number given as the LOS Sensitivity value will be multiplied by 16 to
                                           // arrive to the pulse count number N. The time for detecting LOS will be
                                           // 16 x LOS Sensitivity x Pulse Length (488ns).
                                           // The maximum time is 16 x 256 x 488ns = 2ms
                                           // <nl> Possible Values: 1-256
                                           // <nl> Example Value:
                                           // LOS Sensitivity  =  128     (16 x 128 x 488ns = 1ms)
  OTX_UINT8               nLosRecovery;    // @field Sets the sensitivity level for the Loss Of Signal (LOS) recovery.
                                           // A LOS alarm will be cleared if in the incoming data a number of Pulses (N)
                                           // occur during the time period set with the LOS Recovery parameter. The time
                                           // interval starts with the first detected pulse transition and a pulse
                                           // count is incremented with every received pulse. If the pulse number is
                                           // greater than the number N set with this parameter, the LOS alarm will be cleared.
                                           // <nl> Possible Values: 1-256
  OTX_UINT16              nLineLength;     // @field Specify the length of the T1 line cable in meters. This information
                                           // is used by the T1 transceivers to adjust the transmit pulse masks according
                                           // to the line length.
                                           // <nl> Possible Values: 0-65535
  OTX_BOOL                bHighTxPower;    // @field When OTX_TRUE the pulses on the line interface will be sent with
                                           // high power. If OTX_FALSE the pulses will be sent with low power.
  OTX_BOOL                bReceiveEqualizer;// @field The Receive Equalizer can be used match the 18dB ITU-T I.431
                                           // requirement. When the Equalizer is turned off (OTX_FALSE), the Receiver is 6dB, when
                                           // turned on, the Receiver is 18dB.
  OTX_UINT32              bfClearChannels; // @field Mark any channels designated as clear channels (64 kbit/s) with bit
                                           // robbed signaling is used. Only meaningful if CAS_BR signaling mode is selected.
                                           // The value in this parameter contains a bit pattern, where a '1' indicates a
                                           // clear channel. The least significant bit corresponds to Time-Slot 0 (TS0),
                                           // bit 1 to TS1, etc.
};

typedef struct OtxT1ConfigOptionsS OtxT1ConfigOptionsS;


/*******************************************************************************
  E1 Specfic Configuration Types
*******************************************************************************/

/*******************************************************************************
   @enum
     E1 Line Codes
   @xref
     <t OtxE1ConfigOptionsS>
     <f OtxE1SetConfiguration>
*******************************************************************************/
enum OtxE1LineCodeE {
  OTX_E1_LC_AMI  = 0,   // @emem Alternate Mark Inversion (AMI) Line Coding
  OTX_E1_LC_HDB3 = 1    // @emem High Density Bipolar 3 (HDB3) Line Coding
};

typedef enum OtxE1LineCodeE OtxE1LineCodeE;


/*******************************************************************************
   @enum
     E1 Frame Formats
   @xref
     <t OtxE1ConfigOptionsS>
     <f OtxE1SetConfiguration>
*******************************************************************************/
enum OtxE1FrameFormatE {
  OTX_E1_FF_DOUBLE_FRAME         = 0,   // @emem Double Frame
  OTX_E1_FF_CRC4_MULTIFRAME      = 1,   // @emem CRC4 Multiframe
  OTX_E1_FF_CRC4_MULTIFRAME_G706 = 2    // @emem CRC4 Multiframe format with modified CRC4 Multiframe alignment
                                        // algorithm (Interworking according to CCITT G.706 Annex B)
};

typedef enum OtxE1FrameFormatE OtxE1FrameFormatE;


/*******************************************************************************
   @enum
     Set the algorithm that is used to detect the Alarm Indication Signal (AIS).
   @comm
     According to ETX300233 an AIS alarm is detected if the incoming data stream
     contains less than 3 zeros within 512 bits and a Loss of Frame Alignment is
     indicated. The alarm will be cleared if 3 or more zeros will be detected within
     512 bits or a Frame Alignment Signal (FAS) word is found. According to
     ITU-T G.775 an AIS alarm is detected if the incoming data stream contains less
     than 3 zeros within a doubleframe (512 bits) over a two doubleframe period (1024 bits).
     The alarm will be cleared if 3 or more zeros are detected within a doubleframe
     for two consecutive doubleframes.
   @xref
     <t OtxE1ConfigOptionsS>
     <f OtxE1SetConfiguration>
*******************************************************************************/
enum OtxE1AisDetectModeE {
  OTX_E1_AIS_ETS300233 = 0,   // @emem AIS alarm will be detected according to ETS300233
  OTX_E1_AIS_G775      = 1    // @emem AIS alarm will be detected according to CCITT G.775
};

typedef enum OtxE1AisDetectModeE OtxE1AisDetectModeE;



/*******************************************************************************
  @struct
    E1 Line Interface Configuration Options
  @xref
   <f OtxE1SetConfiguration>
   <f OtxE1GetConfiguration>
*******************************************************************************/
struct OtxE1ConfigOptionsS {
  OtxE1LineCodeE      eTxLineCode;         // @field Line Coding in Transmit Direction
  OtxE1LineCodeE      eRxLineCode;         // @field Line Coding in Receive Direction
  OtxE1FrameFormatE   eTxFrameFormat;      // @field Frame Format in Transmit Direction
  OtxE1FrameFormatE   eRxFrameFormat;      // @field Frame Format in Receive Direction
  OtxE1AisDetectModeE eAisDetection;       // @field Alarm Indication Signal (AIS) Detect mode
  OTX_UINT8           bfSiBits;            // @field Set the Si bits (spare bits for international use) transmitted in
                                           // the bit position 1 of time-slot 0 in doubleframe format and in bit the
                                           // 1 of time-slot 0 in frames 13 and 15 in CRC-Multiframe format  Use bit pattern
                                           // marking the corresponding bit position '1' the bit should be turned on.
                                           // Si Frame 13, Si Frame 15: 00, 01, 10, 11.
                                           // For Both bits on nSiBits = 0x3
  OTX_UINT8           bfSaBits;            // @field Set the Sa bits (spare bits for national use) transmitted in bit
                                           // positions 4-8 of time-slot 0 of every other (odd) frame. Use bit pattern
                                           // marking the corresponding bit position '1' the bit should be turned on.
                                           // Sa4, Sa5, Sa6, Sa7, Sa8: 00000, 00001, ..., 11111
                                           // <nl> For all Sa bits on nSaBits = 0x1F
  OTX_BOOL            bExtendedHDB3errorDetect;  // @field Selects the error detection mode. When turned off (OTX_FALSE),
                                           // only double violations are detected. When turned on (OTX_TRUE), 0000 strings
                                           // are detected as code violations as well.
  OTX_BOOL            bAutoRegainMultiframe;// @field When enabled (OTX_TRUE), the receiver will search a new basic-
                                           // and multiframing if more than 914 CRC errors have been detected in a
                                           // time interval of one second.
  OTX_BOOL            bAutoResync;         // @field If OTX_TRUE, the Framer will automatically re-synchronize after
                                           // loss of synchronization. If OTX_FALSE the Framer will not synchronize
                                           // automatically but a re-synchronization procedure must be specifically
                                           // initiated by the software.
  OTX_BOOL            bAutoCrcBitInversion; // @field If OTX_TRUE, all CRC bits of one outgoing sub-multiframe are
                                           // inverted in case a CRC error is flagged for the previous received 
                                           // sub-multiframe.
  OTX_BOOL            bAutoRemoteAlarm;    // @field If OTX_TRUE, a Remote Alarm (Yellow Alarm) will be sent automatically
                                           // in the outgoing data stream when the receiver is out of synchronization.
                                           // The alarm will also be automatically removed when the receiver returns to
                                           // the synchronized state.
  OTX_UINT8           nLosSensitivity;     // @field Sets the sensitivity level for the Loss Of Signal (LOS) detection.
                                           // A LOS alarm will be generated if the incoming data stream has no
                                           // transitions during programmable number (N) of consecutive pulse periods.
                                           // The number given as the LOS Sensitivity value will be multiplied by 16 to
                                           // arrive to the pulse count number N. The time for detecting LOS will be
                                           // 16 x LOS Sensitivity x Pulse Length (488ns).
                                           // The maximum time is 16 x 256 x 488ns = 2ms
                                           // <nl> Possible Values: 1-256
                                           // <nl> Example/Default Value:
                                           // LOS Sensitivity  =  128     (16 x 128 x 488ns = 1ms)
  OTX_UINT8           nLosRecovery;        // @field Sets the sensitivity level for the Loss Of Signal (LOS) recovery.
                                           // A LOS alarm will be cleared if in the incoming data a number of Pulses (N)
                                           // occur during the time period set with the LOS Recovery parameter. The time
                                           // interval starts with the first detected pulse transition and a pulse
                                           // count is incremented with every received pulse. If the pulse number is
                                           // greater than the number N set with this parameter, the LOS alarm will be cleared.
                                           // <nl> Possible Values: 1-256
  OTX_UINT16          nLineLength;         // @field Specify the length of the T1 line cable in meters. This information
                                           // is used by the T1 transceivers to adjust the transmit pulse masks according
                                           // to the line length.
                                           // <nl> Possible Values: 0-65535
  OTX_BOOL            bHighTxPower;        // @field When OTX_TRUE the pulses on the line interface will be sent with
                                           // high power. If OTX_FALSE the pulses will be sent with low power.
  OTX_BOOL            bReceiveEqualizer;   // @field The Receive Equalizer can be used match the 18dB ITU-T I.431
                                           // requirement. When the Equalizer is turned off (OTX_FALSE), the Receiver is -10dB (short haul), when
                                           // turned on (OTX_TRUE), the Receiver is -36dB (long haul).
};

typedef struct OtxE1ConfigOptionsS OtxE1ConfigOptionsS;



/*******************************************************************************
   @enum
     Set the line termination for an E1/T1 span.
   @comm
     This function should be called with the appropriate line termination
     mode which matches the type of cable that is connected to the Line Interface.
     If the Line Interface is used for monitoring and connected in a bridged
     configuration, the High Impedance termination
     mode should be used. If the Line Interface is used for generation, or the board is 
     connected to a -20dB monitor port, then one of the 75 (E1), 100 (T1), or 120 (E1) 
     ohm modes should be used. 

     Setting the termination mode also affects transmit pulse level. The level is
     set to comply with the G.703 and T1.102 standards. I.e.

     <nl>G.703 E1 120 ohm Balanced:   Amplitude=3.0V  (+/- 0.3V)
     <nl>G.703 T1 100 ohm Balanced:   Amplitude=3.0V  (+/- 0.6V)
     <nl>G.703 E1  75 ohm Unbalanced: Amplitude=2.37V (+/- 0.237V)
     
     The default is high impedance (no termination), which is set when the driver loads.
*******************************************************************************/
enum OtxT1E1TermModeE {
  OTX_T1E1_TERM_120_OHM, // @emem Span is terminated for 120 ohm twisted pair (balanced)
  OTX_T1E1_TERM_100_OHM, // @emem Span is terminated for 100 ohm twisted pair (balanced)
  OTX_T1E1_TERM_75_OHM,  // @emem Span is terminated for 75 ohm coax (unbalanced)
  OTX_T1E1_TERM_HIGHZ    // @emem Span is not terminated (Balanced, High impedance, for monitoring)
};

typedef enum OtxT1E1TermModeE OtxT1E1TermModeE;


/*******************************************************************************
  @struct
    T1 Bit robbing data to be transmitted (used in CAS). See
    the <f OtxT1SetRobBitData> function.
  @comm
    Note that the A and B channels are used in frame formats F12, ESF,
    and F72. The C and D channels are only used in the ESF frame
    format.
    The struct holds 24 bits for each channel; The least significant
    bit is transmitted first (in channel 1, frame 1). This 24 bit
    value will be repeated unless <f OtxT1SetRobBitData> is called again
    with a different values in this struct.
  @xref
    <f OtxT1SetRobBitData>
    <f OtxT1GetRobBitData>
*******************************************************************************/
struct OtxT1RobBitDataS {
  OTX_UINT32 chA;  // @field Channel A
  OTX_UINT32 chB;  // @field Channel B
  OTX_UINT32 chC;  // @field Channel C (Only used in ESF format)
  OTX_UINT32 chD;  // @field Channel D (Only used in ESF format)
};

typedef struct OtxT1RobBitDataS OtxT1RobBitDataS;



/*******************************************************************************
  @struct
    Value for the E1 ABCD Bits to be transmitted (used in CAS). See
    the <f OtxE1SetABCDBitData> function.
  @comm
    The A, B, C, and D channels are transmitted in time-slot 16 of a
    E1 multframe.

    The struct holds 30 bits for each channel. The least significant
    bit (bit #0, zero counting) is transmitted first in channel 1,
    frame 1. Bit #15 is then transmitted in channel 16, frame 1.
    Next the bits #1 and #16 are transmitted in channels 2 and 17 in
    frame 2. And Finally, the bits #14 and #29 are transmitted on
    channels 15 and 30 in Frame 15.

    These values will be repeated unless <f OtxE1SetABCDBitData> is called
    again with a different values in this struct.

  @ex Multi-Frame structure for time-slot 16 |

+-----------+-----------+-----------+-----------+     +-----------+
|   TS16    |   TS16    |   TS16    |   TS16    |     |   TS16    |
|  Frame 0  |  Frame 1  |  Frame 2  |  Frame 3  |     |  Frame 15 |
+-----------+-----------+-----------+-----------+ ... +-----------+
| 0000xyxx  | abcd abcd | abcd abcd | abcd abcd |     | abcd abcd |
|           | ch1  ch16 | ch2  ch17 | ch3  ch18 |     | ch15 ch30 |
+-----------+-----------+-----------+-----------+     +-----------+

  @devnote

  When B, C or D are not used, they should have values
         B = 1, C = 0, D = 1.

  It is recommended that the combination of 0000 of A, B, C, and D should
  not be used for signalling purposes for channels 1 to 15.

  @xref
    <f OtxE1SetRobBitData>
    <f OtxE1GetRobBitData>
*******************************************************************************/
struct OtxE1ABCDBitDataS {
  OTX_UINT32 chA;  // @field Channel A
  OTX_UINT32 chB;  // @field Channel B
  OTX_UINT32 chC;  // @field Channel C
  OTX_UINT32 chD;  // @field Channel D
};

typedef struct OtxE1ABCDBitDataS OtxE1ABCDBitDataS;



/*******************************************************************************
  @enum
     E1 time-slot 0 Sa bits (spare bits for national use)
  @xref
    <f OtxE1SetSaBit>
    <f OtxE1GetSaBit>
*******************************************************************************/
enum OtxE1SaBitE {
  OTX_E1_SA4 = 0,       // @emem Sa Bit number 4
  OTX_E1_SA5 = 1,       // @emem Sa Bit number 5
  OTX_E1_SA6 = 2,       // @emem Sa Bit number 6
  OTX_E1_SA7 = 3,       // @emem Sa Bit number 7
  OTX_E1_SA8 = 4        // @emem Sa Bit number 8
};

typedef enum OtxE1SaBitE OtxE1SaBitE;


#ifdef __linux__
#pragma pack()
#else
#pragma pack(pop)
#endif

#ifdef __cplusplus
  } // extern "C"
#endif

/*******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 1998-2001


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

*******************************************************************************/
#endif // OTX_T1E1_DEF_H
